/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.service;

import static org.fortiss.tooling.kernel.ToolingKernelActivator.getService;

import java.util.List;

import org.conqat.lib.commons.collections.UnmodifiableList;
import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.ToolingKernelActivator;
import org.fortiss.tooling.kernel.extension.IPrototypeProvider;
import org.fortiss.tooling.kernel.extension.data.Prototype;
import org.fortiss.tooling.kernel.extension.data.PrototypeCategory;
import org.fortiss.tooling.kernel.utils.ServicesUtils;

/**
 * The prototype service provides registration and access to model element
 * prototypes. It distinguishes between two types of prototypes. Primary
 * prototypes resemble first-class entities or modeling views. All other
 * prototypes are second class. This allows the user interface to provide
 * different handling of these classes of prototypes.
 * 
 * @see IPrototypeProvider
 * @see Prototype
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 002E06C523B6CA456062444F8BDC66FE
 */
public interface IPrototypeService {
	/**
	 * Returns the service instance.
	 * 
	 * @deprecated use {@link ServicesUtils#getService(org.osgi.framework.Bundle, Class)} and
	 *             {@link ServicesUtils#ungetService(org.osgi.framework.Bundle, Class)}, instead.
	 *             See also {@link ToolingKernelActivator#getService(Class)} and
	 *             {@link ToolingKernelActivator#ungetService(Class)}
	 */
	@Deprecated
	public static IPrototypeService getInstance() {
		return getService(IPrototypeService.class);
	}

	/** Returns the list of all registered prototype providers. */
	UnmodifiableList<IPrototypeProvider> getPrototypeProviders();

	/** Returns the list of all prototypes registered any provider. */
	UnmodifiableList<Prototype> getAllPrototypes();

	/** Returns the list of primary prototypes registered any provider. */
	UnmodifiableList<Prototype> getPrimaryPrototypes();

	/**
	 * Returns the list of all prototypes composable with the given model
	 * element class.
	 */
	List<Prototype> getComposablePrototypes(Class<? extends EObject> modelElementType);

	/** Returns all top-level prototypes categories. */
	List<PrototypeCategory> getAllTopLevelPrototypesCategories();
}
